#include <iostream>

#include "createsshkeys.h"

#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <qprocess.h>
#include <qspinbox.h>
#include <qtextedit.h>
#include <kpassdlg.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <kurlrequester.h>
#include <sys/wait.h>


#include <kprocess.h>



CreateSSHKeys::CreateSSHKeys(QWidget* parent, const char* name, bool modal, WFlags fl)
                : CreateSSHKeys_base(parent,name, modal,fl)
{
    // Start filedialog in $HOME/.ssh
    QString startdir = QString(getenv("HOME"));
    startdir.append("/.ssh/");
    keyFile->setURL(startdir);
    
    // Run this slot to set the initial state of the createButton
    slotPasswordUpdated();
}

CreateSSHKeys::~CreateSSHKeys()
{}


void CreateSSHKeys::slotCreate()
{
    QString privkeyfile = keyFile->url();
    QString pubkeyfile = textPublicKeyShow->text();
    QString password = keyPassword->password();
    QString keyType;
    QStringList output;
    
    // Ask to overwrite the private keyfile if it exists
    if ( QFile::exists(privkeyfile) ) {
        int answer = KMessageBox::warningYesNo(this,
            i18n("File %1 already exists. Do you wish to overwrite the file?").arg(privkeyfile),
            i18n("Overwrite file?"));
        if ( answer == KMessageBox::Yes ) {
            QFile::remove(privkeyfile);
        } else {
            return;
        }
    }
    
    // Ask to overwrite the public keyfile if it exists
    if ( QFile::exists(pubkeyfile) ) {
        int answer = KMessageBox::warningYesNo(this,
            i18n("File %1 already exists. Do you wish to overwrite the file?").arg(pubkeyfile),
            i18n("Overwrite file?"));
        if ( answer == KMessageBox::Yes ) {
            QFile::remove(privkeyfile);
        } else {
            return;
        }
    }
    
    // Disable the createButton, so it can only be pushed if all input is validated
    createButton->setEnabled(false);
    
    if ( keyTypeDSA->isChecked() ) {
        keyType = "dsa";
    }
    if ( keyTypeRSA->isChecked() ) {
        keyType = "rsa";
    }
    
    // Setup the ssh-genkey process
    QProcess * p = new QProcess();
    p->addArgument("ssh-keygen");
    
    // Bytes
    p->addArgument("-b");
    p->addArgument(keySize->text());
    
    // File
    p->addArgument("-f");
    p->addArgument(privkeyfile);
    
    // Password
    p->addArgument("-N");
    p->addArgument(password);
    
    // Type
    p->addArgument("-t");
    p->addArgument(keyType);
    
    // Execute and wait for process to finish
    p->start();
    int status;
    waitpid(p->processIdentifier(), &status, 0);
    
    while ( p->canReadLineStdout() ) {
        QString line = QString(p->readLineStdout());
        if ( line != "Generating public/private rsa key pair." ) {
            output.append(line);
        }
    }
    
    // List arguments
    QStringList list = p->arguments();
    QStringList::Iterator it = list.begin();
    while( it != list.end() ) {
        std::cerr << *it;
        ++it;
    }
    
    KMessageBox::information(this, output.join("\n"));
    createButton->setEnabled(true);
}


void CreateSSHKeys::slotExit()
{
    this->close();
}


void CreateSSHKeys::slotKeyFileUpdated()
{
    QString url = keyFile->url();
    textPublicKeyShow->setText(url + ".pub");
}


void CreateSSHKeys::slotUsePassword(bool checked)
{
    if (checked) {
        keyPassword->setEnabled(true);
        keyPasswordAgain->setEnabled(true);
    } else {
        keyPassword->setEnabled(false);
        keyPasswordAgain->setEnabled(false);
    }
}


void CreateSSHKeys::slotPasswordUpdated()
{
    if (usePassword->isChecked()) {
        QString password = QString(keyPassword->password());
        QString passwordagain = QString(keyPasswordAgain->password());
        
        if ( password.length() >= 4 && passwordagain.length() >= 4) {
            if ( password == passwordagain ) {
                textPasswordMatch->setText(i18n("The passwords match"));
                createButton->setEnabled(true);
            } else {
                textPasswordMatch->setText(i18n("The passwords do <b>not</b> match"));
                createButton->setEnabled(false);
            }
        } else {
            textPasswordMatch->setText(i18n("The password must contain at least 4 characters"));
            createButton->setEnabled(false);
        }
    } else {
        createButton->setEnabled(true);
    }
}


#include "createsshkeys.moc"

