

#include "ksshagent.h"
#include "settings.h"
#include "prefs.h"

#include <kconfigdialog.h>
#include <kedittoolbar.h>
#include <kkeydialog.h>
#include <klocale.h>
#include <kstatusbar.h>
#include <kstdaction.h>
#include <ksystemtray.h>


Ksshagent::Ksshagent()
    : KMainWindow( 0, "Ksshagent" ),
      m_view(new KsshagentView(this))
{
    // accept dnd
    setAcceptDrops(true);

    // tell the KMainWindow that this is indeed the main widget
    setCentralWidget(m_view);

    // then, setup our actions
    setupActions();

    // and a status bar
    statusBar()->show();

    // apply the saved mainwindow settings, if any, and ask the mainwindow
    // to automatically save settings if changed: window size, toolbar
    // position, icon size, etc.
    setAutoSaveSettings();

    // allow the view to change the statusbar and caption
    connect(m_view, SIGNAL(signalChangeStatusbar(const QString&)),
            this,   SLOT(changeStatusbar(const QString&)));
    connect(m_view, SIGNAL(signalChangeCaption(const QString&)),
            this,   SLOT(changeCaption(const QString&)));
    
    // Embed in systemtray
    if ( Settings::useSystray() ) {
        KSystemTray * m_systemtray = new KSystemTray(this);
        m_systemtray->setPixmap(KSystemTray::loadIcon("ksshagent"));
        m_systemtray->show();
        m_systemtray->setInactive();
    }
}

Ksshagent::~Ksshagent()
{
    
}

void Ksshagent::setupActions()
{
    createStandardStatusBarAction();
    
    // TODO: Create a new dialog to create ssh keys
    KStdAction::openNew(m_view, SLOT(fileNew()), actionCollection());
    KStdAction::open(m_view, SLOT(fileOpen()), actionCollection());
    KStdAction::redisplay(m_view, SLOT(fileRedisplay()), actionCollection());
    KStdAction::close(m_view, SLOT(fileClose()), actionCollection());
    KStdAction::quit(kapp, SLOT(quit()), actionCollection());

    KStdAction::keyBindings(this, SLOT(optionsConfigureKeys()), actionCollection());
    KStdAction::configureToolbars(this, SLOT(optionsConfigureToolbars()), actionCollection());
    KStdAction::preferences(this, SLOT(optionsPreferences()), actionCollection());

    createGUI();
}


void Ksshagent::optionsConfigureKeys()
{
    // Start the standard key configuration dialog
    KKeyDialog::configure(actionCollection());
}


void Ksshagent::optionsConfigureToolbars()
{
    // use the standard toolbar editor
    KEditToolbar dlg(actionCollection());
    if (dlg.exec())
    {
        // recreate our GUI
        createGUI();
    } 
}


void Ksshagent::newToolbarConfig()
{
    // this slot is called when user clicks "Ok" or "Apply" in the toolbar editor.
    // recreate our GUI, and re-apply the settings (e.g. "text under icons", etc.)
    createGUI();

    applyMainWindowSettings(KGlobal::config(), autoSaveGroup());
}


void Ksshagent::optionsPreferences()
{
	// The preference dialog is derived from prefs-base.ui which is subclassed into Prefs
        KConfigDialog *dialog = new KConfigDialog(this, "settings", Settings::self(), KDialogBase::Swallow);
        dialog->addPage(new Prefs(), i18n("General"), "package_settings");
        connect(dialog, SIGNAL(settingsChanged()), this, SLOT(settingsChanged()));
        dialog->show();
}


void Ksshagent::settingsChanged()
{
    // TODO: Write some code that adds and removes the systray icon on the fly,
    // so you don't have to restart.
}


void Ksshagent::changeStatusbar(const QString& text)
{
    statusBar()->message(text, 2000);
}


void Ksshagent::changeCaption(const QString& text)
{
    setCaption(text);
}


#include "ksshagent.moc"

